# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: file_system.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x66ile_system.proto\x12\x08\x63z.proto*A\n\x0e\x46ileSystemType\x12\t\n\x05LOCAL\x10\x00\x12\x07\n\x03OSS\x10\x01\x12\t\n\x05\x43\x41\x43HE\x10\x02\x12\x07\n\x03RPC\x10\x03\x12\x07\n\x03\x43OS\x10\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'file_system_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FILESYSTEMTYPE']._serialized_start=31
  _globals['_FILESYSTEMTYPE']._serialized_end=96
# @@protoc_insertion_point(module_scope)
